##########################################################################
# GNU GCC compiler flags
##########################################################################
INCLUDE_PATHS += .
INCLUDE_PATHS += $(ROOT_PATH)/cmsis
INCLUDE_PATHS += $(ROOT_PATH)/lib/$(TARGET)/inc

INCLUDES = $(addprefix -I,$(INCLUDE_PATHS))
##########################################################################
# GNU GCC compiler prefix and location
##########################################################################

AS = $(CROSS_COMPILE)gcc
CC = $(CROSS_COMPILE)gcc
CXX= $(CROSS_COMPILE)g++
#LD = $(CROSS_COMPILE)ld
LD = $(CROSS_COMPILE)gcc
SIZE = $(CROSS_COMPILE)size
OBJCOPY = $(CROSS_COMPILE)objcopy
OBJDUMP = $(CROSS_COMPILE)objdump

##########################################################################
# Startup files
##########################################################################

LD_PATH  += $(ROOT_PATH)/lib/$(TARGET)
#LD_SCRIPT = stm32_flash.ld
#LD_SCRIPT = stm32_ram.ld


##########################################################################
# Compiler settings, parameters and flags
##########################################################################

CFLAGS  = -c -Os $(INCLUDES) -Wall -g
CFLAGS += $(MFLAGS) $(addprefix -D,$(DEFINES))
CFLAGS += -ffunction-sections -fdata-sections -fmessage-length=0
CFLAGS += -Wa,-adhlns=$(@:%.o=%.lst)

ASFLAGS = -c -Os $(INCLUDES) -Wall
ASFLAGS+= $(MFLAGS) -D__ASSEMBLY__ -x assembler-with-cpp

#LDFLAGS = -nostartfiles
#LDFLAGS+= -nodefaultlibs -nostdlib
#LDFLAGS+= -specs=nano.specs
#LDFLAGS+= --gc-sections -Map=firmware.map --cref
LDFLAGS+= $(MFLAGS)
LDFLAGS+= -Wl,--gc-sections,-Map=$(OUTFILE).map,--cref
LDLIBS += $(addprefix -L,$(LD_PATH)) $(LD_SCRIPT)
#LDLIBS += -lm

OCFLAGS = --strip-unneeded

BOBJS = $(addprefix $(BLD),$(OBJS))

all: $(BLD) $(OUTFILE).elf

$(BLD):
	mkdir $(BLD)

$(BLD)%.o : %.c
	$(CC) $(CFLAGS) -o $@ $<

$(BLD)%.o : %.cpp
	$(CXX) $(CFLAGS) -fno-rtti -fno-exceptions -o $@ $<
	
$(BLD)%.o : %.s
	$(AS) $(ASFLAGS) -o $@ $<

$(OUTFILE).elf: $(BOBJS)
	$(LD) $(LDFLAGS) -o $(OUTFILE).elf $(BOBJS) $(LDLIBS)
	-@echo ""
	$(SIZE) $(OUTFILE).elf
	-@echo ""
	$(OBJDUMP) -h -S $(OUTFILE).elf > $(OUTFILE).lst
	$(OBJCOPY) $(OCFLAGS) -O binary $(OUTFILE).elf $(OUTFILE).bin
#	$(OBJCOPY) $(OCFLAGS) -O ihex   $(OUTFILE).elf $(OUTFILE).hex
	-@echo ""

clean:
	rm -f $(OUTFILE).elf $(OUTFILE).bin $(OUTFILE).hex
	rm -f $(BLD)*.o $(BLD)*.lst
	rm -f $(OUTFILE).map  *.lst
#	rm -f $(addsuffix /*~,$(VPATH))
#	rm -f $(addsuffix /*~,$(INCLUDE_PATHS))
#	rm -f $(CLEANER)
