#ifndef _COMMON_H
#define _COMMON_H

#include <stdint.h>
#include "irq.h"
#include "core_cm0.h"

#ifndef __IO
#define __IO volatile
#endif // __IO
/// Pro 16-bitové registry nutno pakovat
#define PACKSTRUCT __attribute__((packed))

/** Zde vzniká drobný problém - použít typ enum v bitovém poli jako další
stupeň abstrakce je sice hezké, ale na ARM nepříliš funkční. Default je
překladač nastaven tak, že enum interpretuje jako byte a tak to také zapisuje
do registrů. To ale nefunguje. Lze nastavit -fno-short-enums, ale pak
jsou potíže s knihovnami - binární kód není kompatibilní. Takže je použit
kompromis - v bitovém poli je 32-bit unsigned a přiřazení se dělá z toho
enum typu. Není pak problém ani s 16-bitovým registrem, pokud je struktura
s bitovým polem pakována.
*/
typedef enum {
  RESET = 0, SET
} _ONEBIT;
/// Alias pro (tento) enum v bitovém poli je pak 32-bit unsigned.
typedef uint32_t ONEBIT;

// Stejně je to pro všechny enum, které jsou pak v hlavičkách definovány.

/** @addtogroup Peripheral_memory_map
  * @{
  */

#define FLASH_BASE            ((uint32_t)0x08000000) /*!< FLASH base address in the alias region */
#define SRAM_BASE             ((uint32_t)0x20000000) /*!< SRAM base address in the alias region */
#define PERIPH_BASE           ((uint32_t)0x40000000) /*!< Peripheral base address in the alias region */

/*!< Peripheral memory map */
#define APBPERIPH_BASE        PERIPH_BASE
#define AHBPERIPH_BASE        (PERIPH_BASE + 0x00020000)
#define AHB2PERIPH_BASE       (PERIPH_BASE + 0x08000000)

#define TIM2_BASE             (APBPERIPH_BASE + 0x00000000)
#define TIM3_BASE             (APBPERIPH_BASE + 0x00000400)
#define TIM6_BASE             (APBPERIPH_BASE + 0x00001000)
#define TIM14_BASE            (APBPERIPH_BASE + 0x00002000)
#define RTC_BASE              (APBPERIPH_BASE + 0x00002800)
#define WWDG_BASE             (APBPERIPH_BASE + 0x00002C00)
#define IWDG_BASE             (APBPERIPH_BASE + 0x00003000)
#define SPI2_BASE             (APBPERIPH_BASE + 0x00003800)
#define USART2_BASE           (APBPERIPH_BASE + 0x00004400)
#define I2C1_BASE             (APBPERIPH_BASE + 0x00005400)
#define I2C2_BASE             (APBPERIPH_BASE + 0x00005800)
#define PWR_BASE              (APBPERIPH_BASE + 0x00007000)
#define DAC_BASE              (APBPERIPH_BASE + 0x00007400)
#define CEC_BASE              (APBPERIPH_BASE + 0x00007800)

#define SYSCFG_BASE           (APBPERIPH_BASE + 0x00010000)
#define COMP_BASE             (APBPERIPH_BASE + 0x0001001C)
#define EXTI_BASE             (APBPERIPH_BASE + 0x00010400)
#define ADC1_BASE             (APBPERIPH_BASE + 0x00012400)
#define ADC_BASE              (APBPERIPH_BASE + 0x00012708)
#define TIM1_BASE             (APBPERIPH_BASE + 0x00012C00)
#define SPI1_BASE             (APBPERIPH_BASE + 0x00013000)
#define USART1_BASE           (APBPERIPH_BASE + 0x00013800)
#define TIM15_BASE            (APBPERIPH_BASE + 0x00014000)
#define TIM16_BASE            (APBPERIPH_BASE + 0x00014400)
#define TIM17_BASE            (APBPERIPH_BASE + 0x00014800)
#define DBGMCU_BASE           (APBPERIPH_BASE + 0x00015800)

#define DMA1_BASE             (AHBPERIPH_BASE + 0x00000000)
#define DMA1_Channel1_BASE    (DMA1_BASE + 0x00000008)
#define DMA1_Channel2_BASE    (DMA1_BASE + 0x0000001C)
#define DMA1_Channel3_BASE    (DMA1_BASE + 0x00000030)
#define DMA1_Channel4_BASE    (DMA1_BASE + 0x00000044)
#define DMA1_Channel5_BASE    (DMA1_BASE + 0x00000058)
#define RCC_BASE              (AHBPERIPH_BASE + 0x00001000)
#define FLASH_R_BASE          (AHBPERIPH_BASE + 0x00002000) /*!< FLASH registers base address */
#define OB_BASE               ((uint32_t)0x1FFFF800)        /*!< FLASH Option Bytes base address */
#define CRC_BASE              (AHBPERIPH_BASE + 0x00003000)
#define TSC_BASE              (AHBPERIPH_BASE + 0x00004000)

#define GPIOA_BASE            (AHB2PERIPH_BASE + 0x00000000)
#define GPIOB_BASE            (AHB2PERIPH_BASE + 0x00000400)
#define GPIOC_BASE            (AHB2PERIPH_BASE + 0x00000800)
#define GPIOD_BASE            (AHB2PERIPH_BASE + 0x00000C00)
#define GPIOF_BASE            (AHB2PERIPH_BASE + 0x00001400)


#define TGPIOA               ((GPIO_Type *) GPIOA_BASE)
#define TGPIOB               ((GPIO_Type *) GPIOB_BASE)
#define TGPIOC               ((GPIO_Type *) GPIOC_BASE)
#define TGPIOD               ((GPIO_Type *) GPIOD_BASE)
#define TGPIOF               ((GPIO_Type *) GPIOF_BASE)

#define  TRCC_AHBENR_GPIOAEN   ((uint32_t)0x00020000)        /*!< GPIOA clock enable */
#define  TRCC_AHBENR_GPIOBEN   ((uint32_t)0x00040000)        /*!< GPIOB clock enable */
#define  TRCC_AHBENR_GPIOCEN   ((uint32_t)0x00080000)        /*!< GPIOC clock enable */
#define  TRCC_AHBENR_GPIODEN   ((uint32_t)0x00100000)        /*!< GPIOD clock enable */
#define  TRCC_AHBENR_GPIOFEN   ((uint32_t)0x00400000)        /*!< GPIOF clock enable */


#endif // _COMMON_H
