#ifndef SOFTINTR_H
#define SOFTINTR_H
#include "irq.h"

/**
 * @file
 * @brief Softwarového přerušení pro STM32F0.
 */

/// Vyčíslení použité instance
enum SoftIntrNumber {
  SoftIntrNumber0 = 0,  //!< 0
  SoftIntrNumber1,      //!< 1
  SoftIntrNumber2,      //!< 2
  MaxSoftIntrs          //!< Maximum pro tabulky
};
/**
 * @class SoftIntr
 * @brief Softwarového přerušení pro STM32F0.
 * 
 * Můžeme vytvořit 3 instance tt. třídy, resp. jejího potomka, které se budou lišit použitým
 * vektorem (máme 3 volné vektory). Přerušení se vyvolá metodou exec(), kód pak probíhá
 * v přetížené metodě run(). Manipulace s přerušením používá CMSIS funkce NVIC_SetPendingIRQ(),
 * které vyvolá přerušení bezprostředně. NVIC_ClearPendingIRQ() vlastně použito být nemusí,
 * bit se maže automaticky vstupem do přerušení.
 */
class SoftIntr {
  public:
    /// Konstruktor @param n instance
    SoftIntr (const SoftIntrNumber n);
    /// Vyvolej přerušení
    void         exec  (void);
    /// V potomku si napiš vlastní kód pro obsluhu
    virtual void run   (void) = 0;
  private:
    /// použité přerušení (viz tabulka)
    const IRQn_Type  N;
};

#endif // SOFTINTR_H
